@@ -13,15 +13,56 @@ if IsWin
             cmd = 'g++ -g -o libptbdrawtext_ftgl.dll -fno-exceptions -shared -LC:\gstreamer\1.0\msvc_x86\lib -I. -IC:\gstreamer\1.0\msvc_x86\include\freetype2 -IC:\gstreamer\1.0\msvc_x86\include\fontconfig libptbdrawtext_ftgl.cpp qstringqcharemulation.cpp OGLFT.cpp -lopengl32 -lglu32 -lfontconfig -lfreetype';
             name = 'libptbdrawtext_ftgl.dll';
         end
+        warning('Drawtext plugins built with Octave - or rather MinGW g++ - only work with Octave, not with Matlab! Use Matlab / MSVC for a build that works everywhere!');
     else
-        % 64-Bit build with MSVC 2010, done from a separate MS-DOS batch script:
+        % 64-Bit build with MSVC 2019, done from a separate MS-DOS batch script:
         cmd = 'buildftglpluginformsvc64Bit.bat';
         name = 'libptbdrawtext_ftgl64.dll';
+
+        % Does our own self-buit fontconfig.lib import file already exist?
+        if ~exist('fontconfig.lib', 'file')
+            % No. Need to build it ourselves, as the one shipping with
+            % GStreamer 1.20.5 is broken!
+            fprintf('Generating our own fontconfig-1.lib import library now...\n');
+
+            % 1. Extract fontconfig function definitions from GStreamer's
+            % fontconfig-1.dll into a file. Our build batch file does this
+            % as a side-effect:
+            [rc, ret] = system(cmd);
+            if rc ~= 0 || ~exist('fontconfig-1.functions', 'file')
+                disp(ret);
+                error('Failed to parse out fontconfig function definitions!');
+            end
+
+            % Parse the file and turn it into a linker .def file:
+            f = fopen('fontconfig-1.functions');
+            fo=fopen('fontconfig-1.def', 'wt');
+            fprintf(fo, 'EXPORTS\n');
+            while ~feof(f)
+                l = fgetl(f);
+                lo = l(strfind(l, 'Fc'):end);
+                if ~isempty(lo)
+                    fprintf(fo, '%s\n', lo);
+                end
+            end
+            fclose(f);
+            fclose(fo);
+
+            if ~exist('fontconfig-1.def', 'file')
+                disp(ret);
+                error('Failed to generate fontconfig-1.def file!');
+            end
+
+            % Ok, got the file. A successive 2nd invocation of our batch
+            % file will turn that fontconfig-1.def file into a
+            % fontconfig-1.lib library import definition file for use by
+            % the linker while building our FTGL plugin, to finally use the
+            % right fontconfig-1.dll library from GStreamer in the end.
+        end
     end
 end
 
 if IsOSX
-    %cmd = 'g++ -g -DHAVE_OPENGL_DIR -I.  -I/usr/X11/include/ -I/usr/X11/include/freetype2/ -L/usr/X11/lib/ -framework OpenGL -l fontconfig -l freetype -dynamiclib -o libptbdrawtext_ftgl64.dylib libptbdrawtext_ftgl.cpp qstringqcharemulation.cpp OGLFT.cpp';
     cmd = 'g++ -g -DHAVE_OPENGL_DIR -I.  -I/Library/Frameworks/GStreamer.framework/Versions/Current/include/ -I/Library/Frameworks/GStreamer.framework/Versions/Current/include/freetype2/ -L/Library/Frameworks/GStreamer.framework/Versions/Current/lib/ -framework OpenGL -l fontconfig -l freetype -dynamiclib -o libptbdrawtext_ftgl64.dylib libptbdrawtext_ftgl.cpp qstringqcharemulation.cpp OGLFT.cpp';
     name = 'libptbdrawtext_ftgl64.dylib';
 end
@@ -52,4 +93,8 @@ if ~isempty(cmd)
     end
 end
 
+if IsWin
+    delete('fontconfig-1.functions');
+end
+
 return;
