@@ -1,4 +1,4 @@
-function VRHMDDemo1(doSeparateEyeRender, multiSample, fountain, checkerboard)
+function VRHMDDemo1(multiSample, fountain, checkerboard, gpumeasure, debugView, doSeparateEyeRender)
 % VRHMDDemo1 -- Show 3D stereo display via MOGL OpenGL on a VR headset.
 %
 % This demo shows how to use Psychtoolbox PsychVRHMD() driver to display
@@ -8,10 +8,31 @@ function VRHMDDemo1(doSeparateEyeRender, multiSample, fountain, checkerboard)
 % case in "Happy teapot land". Obviously, this demo will only work if you have
 % one of the supported HMDs connected to your machine.
 %
-% Usage: VRHMDDemo1([doSeparateEyeRender][, multiSample=1][, fountain=1][, checkerboard=0]);
+% Usage: VRHMDDemo1([multiSample=4][, fountain=1][, checkerboard=0][, gpumeasure=0][, debugView=0][, doSeparateEyeRender]);
 %
 % Optional parameters:
 %
+% 'multiSample' if set to a non-zero value will enable multi-sample
+% anti-aliasing. Can increase quality, but also increases GPU load.
+%
+% 'fountain' if set to 1 will also emit a particle fountain from the nozzle
+% of the teapot. Nicer, but higher gpu load.
+%
+% 'checkerboard' if set to 1, show a checkerboard texture instead of the regular
+% teapot.
+%
+% 'gpumeasure' if set to 1 will use Screen's builtin functions for measurement of
+% gpu time spent on post-processing the VR images. This excludes actual 3D rendering,
+% as time spent between Screen('BeginOpenGL') and Screen('EndOpenGL') won't be measured.
+% Note that some VR runtimes have trouble with gpumeasure, as it clashes with their own
+% builtin measurement mechanisms, so gpumeasure may cause errors or error messages.
+% E.g., a major offender as of December 2022 is the SteamVR OpenXR runtime, which will
+% cause flooding of the console with OpenGL error messages - Reason unknown, but was not
+% resolvable in hours of work.
+%
+% 'debugView' if set to 1, also display the VR content in a Psychtoolbox onscreen
+% window. May impact performance!
+%
 % 'doSeparateEyeRender' if set to 1, perform per eye render passes in an optimized
 % order, as recommended by the HMD driver, and query the per eye camera matrices
 % individually during each render pass to optimize for head tracking prediction
@@ -19,12 +40,10 @@ function VRHMDDemo1(doSeparateEyeRender, multiSample, fountain, checkerboard)
 % and render in a potentially non-optimized order. The latter is a bit simpler, but
 % potentially less accurate, causing additional motion artifacts. If the setting is
 % omitted then the underlying HMD driver will be asked for the optimal value.
-%
-% 'multiSample' if set to a non-zero value will enable multi-sample
-% anti-aliasing. Can increase quality but also increases GPU load.
-%
-% 'fountain' if set to 1 will also emit a particle fountain from the nozzle
-% of the teapot. Nicer, but higher gpu load.
+% Note that none of the modern VR runtimes supports a doSeparateEyeRender setting of 1
+% in any meaningful way. Only the - long dead - original OculusVR v0.5 runtime on the
+% old Oculus Rift DK1 and DK2 supported this for improved quality. The correct setting
+% therefore is always 0 on modern systems, and this is what will be chosen by default.
 %
 % Press any key to end the demo.
 %
@@ -42,24 +61,38 @@ function VRHMDDemo1(doSeparateEyeRender, multiSample, fountain, checkerboard)
 % 10-Sep-2015  mk  Written. Derived from DrawDots3DDemo.m
 
 % GL data structure needed for all OpenGL demos:
-global GL;
+global GL; %#ok<GVMIS> 
 
-if nargin < 1 || isempty(doSeparateEyeRender)
-  doSeparateEyeRender = [];
+if nargin < 1 || isempty(multiSample)
+  multiSample = 4;
 end
 
-if nargin < 2 || isempty(multiSample)
-  multiSample = 8;
-end
-
-if nargin < 3 || isempty(fountain)
+if nargin < 2 || isempty(fountain)
   fountain = 1;
 end
 
-if nargin < 4  || isempty(checkerboard)
+if nargin < 3  || isempty(checkerboard)
   checkerboard = 0;
 end
 
+if nargin < 4  || isempty(gpumeasure)
+  gpumeasure = 0;
+end
+
+if nargin < 5 || isempty(debugView)
+  debugView = 0;
+end
+
+if debugView
+  debugView = 'DebugDisplay';
+else
+  debugView = '';
+end
+
+if nargin < 6 || isempty(doSeparateEyeRender)
+  doSeparateEyeRender = [];
+end
+
 % Default setup:
 PsychDefaultSetup(2);
 
@@ -73,7 +106,8 @@ try
 
   % Setup the HMD and open and setup the onscreen window for VR display:
   PsychImaging('PrepareConfiguration');
-  hmd = PsychVRHMD('AutoSetupHMD', 'Tracked3DVR', 'LowPersistence TimeWarp FastResponse DebugDisplay', 0);
+  % We do collect timestamps for benchmarking, but don't require them to be especially precise or trustworthy:
+  hmd = PsychVRHMD('AutoSetupHMD', 'Tracked3DVR', ['NoTimingSupport NoTimestampingSupport' debugView], 0);
   if isempty(hmd)
     fprintf('No VR-HMD available, giving up!\n');
     return;
@@ -147,9 +181,6 @@ try
   % standard 2D drawing functions of Screen and will check for OpenGL errors.
   Screen('EndOpenGL', win);
 
-  % Number of random dots, whose positions are computed in Matlab on CPU:
-  ndots = 100;
-
   % Number of fountain particles whose positions are computed on the GPU:
   nparticles = 10000;
 
@@ -272,7 +303,7 @@ try
   end
 
   telapsed = 0;
-  fcount = 0;
+  fcount = 1;
 
   % Allocate for up to 1000 seconds at nominal HMD fps:
   fps = Screen('FrameRate', win);
@@ -285,10 +316,7 @@ try
   % Make sure all keys are released:
   KbReleaseWait;
 
-  Priority(MaxPriority(win));
-
-  % Get duration of a single frame:
-  ifi = Screen('GetFlipInterval', win);
+%  Priority(MaxPriority(win));
 
   globalPos = [0, 0, 3];
   heading = 0;
@@ -298,8 +326,11 @@ try
   HideCursor(screenid);
   [xo, yo] = GetMouse(screenid);
 
+  % Only accept ESCape key for stopping this demo:
+  RestrictKeysForKbCheck(KbName('ESCAPE'));
+
   % Initial flip to sync us to VBL and get start timestamp:
-  [vbl, onset] = Screen('Flip', win);
+  [vbl, onset(fcount)] = Screen('Flip', win);
   tstart = vbl;
 
   % VR render loop: Runs until keypress:
@@ -339,9 +370,11 @@ try
     % by the PsychGetPositionYawMatrix() helper function:
     state = PsychVRHMD('PrepareRender', hmd, globalHeadPose);
 
-    % Start rendertime measurement on GPU: 'gpumeasure' will be 1 if
+    % Optionally start rendertime measurement on GPU: 'gpumeasure' will be 1 if
     % this is supported by the current GPU + driver combo:
-    gpumeasure = Screen('GetWindowInfo', win, 5);
+    if gpumeasure
+      gpumeasure = Screen('GetWindowInfo', win, 5);
+    end
 
     % We render the scene separately for each eye:
     for renderPass = 0:1
@@ -441,7 +474,7 @@ try
     % Head position tracked?
     if ~bitand(state.tracked, 2) && ~checkerboard
       % Nope, user out of cameras view frustum. Tell it like it is:
-      DrawFormattedText(win, 'Vision based tracking lost\nGet back into the cameras field of view!', 'center', 'center', [1 0 0]);
+      %DrawFormattedText(win, 'Vision based tracking lost\nGet back into the cameras field of view!', 'center', 'center', [1 0 0]);
     end
 
     % Stimulus ready. Show it on the HMD. We don't clear the color buffer here,
@@ -470,6 +503,8 @@ try
   end
 
   % Cleanup:
+  fprintf('Exiting display loop...\n');
+  RestrictKeysForKbCheck([]);
   Priority(0);
   ShowCursor(screenid);
   sca;
@@ -478,8 +513,12 @@ try
   fps = fcount / (vbl - tstart);
   gpudur = gpudur(1:fcount);
   fprintf('Average framerate was %f fps. Average GPU rendertime per frame = %f msec.\n', fps, 1000 * mean(gpudur));
-  plot(1000 * gpudur);
-  title('GPU processing time per frame [msecs]: (Often wrong on buggy OSX!)');
+
+  if gpumeasure
+      plot(1000 * gpudur);
+      title('GPU processing time per frame [msecs]:');
+  end
+
   figure;
   plot(1000 * diff(onset(1:fcount)));
   title('Time delta between presentation of frames [msecs]: ');
