@@ -80,11 +80,10 @@ function PsychPaidSupportAndServices(mininag)
 % other words, it contributes to / acts as an insurance that allows
 % Psychtoolbox to be around and in good shape years into the future.
 %
-% To clarify: Psychtoolbox itself will stay freely downloadable and usable
-% by anyone, and stay fully accessible as open-source software, by anyone
-% for any purpose. Anybody able and willing to contribute code and ideas of
-% sufficiently high quality is invited to contribute to the open-source
-% code, the documentation on our public community website and Wiki
+% To clarify: Psychtoolbox itself will stay open-source software. Anybody
+% able and willing to contribute code and ideas of sufficiently high quality
+% is invited to contribute to the open-source code, the documentation on our
+% public community website and Wiki
 % http://psychtoolbox.org, and to participate and help each other on the
 % public community forum https://psychtoolbox.discourse.group.
 %
@@ -113,12 +112,11 @@ if exist('mininag', 'var') && (mininag > 0)
     fprintf('\n\n');
 
     if mininag == 1
-        % Advertise our user survey 2022:
-        fprintf('ADDITIONAL NEWS: The Psychtoolbox user survey 2022 is underway!\n');
+        % Advertise our user survey 2022/2023:
+        fprintf('ADDITIONAL NEWS: Psychtoolbox user survey 2022/2023!\n');
         fprintf('\n');
-        fprintf('In recent years the Psychtoolbox Team has worked hard to improve Psychtoolbox itself and its services.\n');
-        fprintf('We kindly ask you to fill out this survey, so that we can better understand the needs and wishes of our\n');
-        fprintf('community. Your answers will greatly help us to shape the future of Psychtoolbox.\n');
+        fprintf('We kindly ask you to fill out our user survey, so that we can better understand the needs\n');
+        fprintf('and wishes of our community. Your answers will greatly help us to shape the future of Psychtoolbox.\n');
         fprintf('\n');
         fprintf('The survey is conducted by our host for all commercial services and matters, the\n');
         fprintf('“Medical Innovations Incubator GmbH, Tuebingen, Germany”.\n');
@@ -148,7 +146,12 @@ more off;
 
 % Check if user wants to file a support request:
 fprintf('\n\n');
-answer = input('Do you need paid support now and have an active license key [y/n]? ', 's');
+
+answer = '';
+while length(answer)~=1 || ~ismember(answer, ['y', 'n'])
+    answer = strtrim(input('Do you need paid support now and have an active license key [y/n]? ', 's'));
+end
+
 if ~strcmpi(answer, 'y')
     fprintf('A community membership with paid support can be bought as described above. Bye.\n');
     return;
